package com.jiuqi.util.json;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.util.json.IValueConvertor;
import com.jiuqi.util.json.JSONArray;
import com.jiuqi.util.json.JSONException;
import com.jiuqi.util.json.JSONString;
import com.jiuqi.util.json.JSONTokener;
import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeSet;

public class JSONObject {
  private IValueConvertor valueConvertor;
  
  protected HashMap myHashMap;
  
  public void setValueConvertor(IValueConvertor valueConvertor) {
    this.valueConvertor = valueConvertor;
  }
  
  public static final Object NULL = new Null(null);
  
  public JSONObject() {
    this.myHashMap = new HashMap<Object, Object>();
  }
  
  public JSONObject(com.jiuqi.util.json.JSONObject jo, String[] names) throws JSONException {
    this();
    for (int i = 0; i < names.length; i++)
      putOpt(names[i], jo.opt(names[i])); 
  }
  
  public JSONObject(JSONTokener x) throws JSONException {
    this();
    if (x.nextClean() != '{')
      throw x.syntaxError("A JSONObject text must begin with '{'"); 
    while (true) {
      char c = x.nextClean();
      switch (c) {
        case '\000':
          throw x.syntaxError("A JSONObject text must end with '}'");
        case '}':
          return;
      } 
      x.back();
      String key = x.nextValue().toString();
      c = x.nextClean();
      if (c == '=') {
        if (x.next() != '>')
          x.back(); 
      } else if (c != ':') {
        throw x.syntaxError("Expected a ':' after a key");
      } 
      put(key, x.nextValue());
      switch (x.nextClean()) {
        case ',':
        case ';':
          if (x.nextClean() == '}')
            return; 
          x.back();
          continue;
        case '}':
          return;
      } 
      break;
    } 
    throw x.syntaxError("Expected a ',' or '}'");
  }
  
  public JSONObject(Map<?, ?> map) {
    this.myHashMap = (map == null) ? new HashMap<Object, Object>() : new HashMap<Object, Object>(map);
  }
  
  public JSONObject(Object bean) {
    this();
    Class<?> klass = bean.getClass();
    Method[] methods = klass.getMethods();
    for (int i = 0; i < methods.length; i++) {
      try {
        Method method = methods[i];
        String name = method.getName();
        String key = "";
        if (name.startsWith("get")) {
          key = name.substring(3);
        } else if (name.startsWith("is")) {
          key = name.substring(2);
        } 
        if (key.length() > 0 && Character.isUpperCase(key.charAt(0)) && (
          method.getParameterTypes()).length == 0) {
          if (key.length() == 1) {
            key = key.toLowerCase();
          } else if (!Character.isUpperCase(key.charAt(1))) {
            key = String.valueOf(key.substring(0, 1).toLowerCase()) + 
              key.substring(1);
          } 
          put(key, method.invoke(bean, null));
        } 
      } catch (Exception exception) {}
    } 
  }
  
  public JSONObject(Object object, String[] names) {
    this();
    Class<?> c = object.getClass();
    for (int i = 0; i < names.length; i++) {
      String name = names[i];
      try {
        Field field = c.getField(name);
        Object value = field.get(object);
        put(name, value);
      } catch (Exception exception) {}
    } 
  }
  
  public JSONObject(String source) throws JSONException {
    this(new JSONTokener(source));
  }
  
  public com.jiuqi.util.json.JSONObject accumulate(String key, Object value) throws JSONException {
    testValidity(value);
    Object o = opt(key);
    if (o == null) {
      put(key, (value instanceof JSONArray) ? (new JSONArray()).put(value) : 
          value);
    } else if (o instanceof JSONArray) {
      ((JSONArray)o).put(value);
    } else {
      put(key, (new JSONArray()).put(o).put(value));
    } 
    return this;
  }
  
  public com.jiuqi.util.json.JSONObject append(String key, Object value) throws JSONException {
    testValidity(value);
    Object o = opt(key);
    if (o == null) {
      put(key, (new JSONArray()).put(value));
    } else if (o instanceof JSONArray) {
      put(key, ((JSONArray)o).put(value));
    } else {
      throw new JSONException("JSONObject[" + key + 
          "] is not a JSONArray.");
    } 
    return this;
  }
  
  public static String doubleToString(double d) {
    if (Double.isInfinite(d) || Double.isNaN(d))
      return "null"; 
    String s = Double.toString(d);
    if (s.indexOf('.') > 0 && s.indexOf('e') < 0 && s.indexOf('E') < 0) {
      while (s.endsWith("0"))
        s = s.substring(0, s.length() - 1); 
      if (s.endsWith("."))
        s = s.substring(0, s.length() - 1); 
    } 
    return s;
  }
  
  public Object get(String key) throws JSONException {
    Object o = opt(key);
    if (o == null)
      throw new JSONException("JSONObject[" + quote(key) + "] not found."); 
    return o;
  }
  
  public boolean getBoolean(String key) throws JSONException {
    Object o = get(key);
    if (o.equals(Boolean.FALSE) || (
      o instanceof String && ((String)o)
      .equalsIgnoreCase("false")))
      return false; 
    if (o.equals(Boolean.TRUE) || (
      o instanceof String && ((String)o)
      .equalsIgnoreCase("true")))
      return true; 
    throw new JSONException("JSONObject[" + quote(key) + 
        "] is not a Boolean.");
  }
  
  public double getDouble(String key) throws JSONException {
    Object o = get(key);
    try {
      return (o instanceof Number) ? ((Number)o).doubleValue() : 
        Double.valueOf((String)o).doubleValue();
    } catch (Exception exception) {
      throw new JSONException("JSONObject[" + quote(key) + 
          "] is not a number.");
    } 
  }
  
  public int getInt(String key) throws JSONException {
    Object o = get(key);
    return (o instanceof Number) ? ((Number)o).intValue() : 
      (int)getDouble(key);
  }
  
  public JSONArray getJSONArray(String key) throws JSONException {
    Object o = get(key);
    if (o instanceof JSONArray)
      return (JSONArray)o; 
    throw new JSONException("JSONObject[" + quote(key) + 
        "] is not a JSONArray.");
  }
  
  public com.jiuqi.util.json.JSONObject getJSONObject(String key) throws JSONException {
    Object o = get(key);
    if (o instanceof com.jiuqi.util.json.JSONObject)
      return (com.jiuqi.util.json.JSONObject)o; 
    throw new JSONException("JSONObject[" + quote(key) + 
        "] is not a JSONObject.");
  }
  
  public long getLong(String key) throws JSONException {
    Object o = get(key);
    return (o instanceof Number) ? ((Number)o).longValue() : 
      (long)getDouble(key);
  }
  
  public static String[] getNames(com.jiuqi.util.json.JSONObject jo) {
    int length = jo.length();
    if (length == 0)
      return null; 
    Iterator<String> i = jo.keys();
    String[] names = new String[length];
    int j = 0;
    while (i.hasNext()) {
      names[j] = i.next();
      j++;
    } 
    return names;
  }
  
  public static String[] getNames(Object object) {
    if (object == null)
      return null; 
    Class<?> klass = object.getClass();
    Field[] fields = klass.getFields();
    int length = fields.length;
    if (length == 0)
      return null; 
    String[] names = new String[length];
    for (int i = 0; i < length; i++)
      names[i] = fields[i].getName(); 
    return names;
  }
  
  public String getString(String key) throws JSONException {
    return get(key).toString();
  }
  
  public boolean has(String key) {
    return this.myHashMap.containsKey(key);
  }
  
  public boolean isNull(String key) {
    return NULL.equals(opt(key));
  }
  
  public Iterator keys() {
    return this.myHashMap.keySet().iterator();
  }
  
  public int length() {
    return this.myHashMap.size();
  }
  
  public JSONArray names() {
    JSONArray ja = new JSONArray();
    Iterator keys = keys();
    while (keys.hasNext())
      ja.put(keys.next()); 
    return (ja.length() == 0) ? null : ja;
  }
  
  public static String numberToString(Number n) throws JSONException {
    if (n == null)
      throw new JSONException("Null pointer"); 
    testValidity(n);
    String s = n.toString();
    if (s.indexOf('.') > 0 && s.indexOf('e') < 0 && s.indexOf('E') < 0) {
      while (s.endsWith("0"))
        s = s.substring(0, s.length() - 1); 
      if (s.endsWith("."))
        s = s.substring(0, s.length() - 1); 
    } 
    return s;
  }
  
  public Object opt(String key) {
    return (key == null) ? null : this.myHashMap.get(key);
  }
  
  public boolean optBoolean(String key) {
    return optBoolean(key, false);
  }
  
  public boolean optBoolean(String key, boolean defaultValue) {
    try {
      return getBoolean(key);
    } catch (Exception exception) {
      return defaultValue;
    } 
  }
  
  public com.jiuqi.util.json.JSONObject put(String key, Collection value) throws JSONException {
    put(key, new JSONArray(value));
    return this;
  }
  
  public double optDouble(String key) {
    return optDouble(key, Double.NaN);
  }
  
  public double optDouble(String key, double defaultValue) {
    try {
      Object o = opt(key);
      return (o instanceof Number) ? ((Number)o).doubleValue() : (
        new Double((String)o)).doubleValue();
    } catch (Exception exception) {
      return defaultValue;
    } 
  }
  
  public int optInt(String key) {
    return optInt(key, 0);
  }
  
  public int optInt(String key, int defaultValue) {
    try {
      return getInt(key);
    } catch (Exception exception) {
      return defaultValue;
    } 
  }
  
  public JSONArray optJSONArray(String key) {
    Object o = opt(key);
    return (o instanceof JSONArray) ? (JSONArray)o : null;
  }
  
  public com.jiuqi.util.json.JSONObject optJSONObject(String key) {
    Object o = opt(key);
    return (o instanceof com.jiuqi.util.json.JSONObject) ? (com.jiuqi.util.json.JSONObject)o : null;
  }
  
  public long optLong(String key) {
    return optLong(key, 0L);
  }
  
  public long optLong(String key, long defaultValue) {
    try {
      return getLong(key);
    } catch (Exception exception) {
      return defaultValue;
    } 
  }
  
  public String optString(String key) {
    return optString(key, "");
  }
  
  public String optString(String key, String defaultValue) {
    Object o = opt(key);
    return (o != null) ? o.toString() : defaultValue;
  }
  
  public com.jiuqi.util.json.JSONObject put(String key, boolean value) throws JSONException {
    put(key, value ? Boolean.TRUE : Boolean.FALSE);
    return this;
  }
  
  public com.jiuqi.util.json.JSONObject put(String key, double value) throws JSONException {
    put(key, new Double(value));
    return this;
  }
  
  public com.jiuqi.util.json.JSONObject put(String key, int value) throws JSONException {
    put(key, new Integer(value));
    return this;
  }
  
  public com.jiuqi.util.json.JSONObject put(String key, long value) throws JSONException {
    put(key, new Long(value));
    return this;
  }
  
  public com.jiuqi.util.json.JSONObject put(String key, Map value) throws JSONException {
    put(key, new com.jiuqi.util.json.JSONObject(value));
    return this;
  }
  
  public com.jiuqi.util.json.JSONObject put(String key, Object value) throws JSONException {
    if (key == null)
      throw new JSONException("Null key."); 
    if (value != null) {
      testValidity(value);
      this.myHashMap.put(key, value);
    } else {
      remove(key);
    } 
    return this;
  }
  
  public com.jiuqi.util.json.JSONObject putOpt(String key, Object value) throws JSONException {
    if (key != null && value != null)
      put(key, value); 
    return this;
  }
  
  public static String quote(String string) {
    if (string == null || string.length() == 0)
      return "\"\""; 
    char c = Character.MIN_VALUE;
    int len = string.length();
    StringBuffer sb = new StringBuffer(len + 4);
    sb.append('"');
    for (int i = 0; i < len; i++) {
      char b = c;
      c = string.charAt(i);
      switch (c) {
        case '"':
        case '\\':
          sb.append('\\');
          sb.append(c);
          break;
        case '/':
          if (b == '<')
            sb.append('\\'); 
          sb.append(c);
          break;
        case '\b':
          sb.append("\\b");
          break;
        case '\t':
          sb.append("\\t");
          break;
        case '\n':
          sb.append("\\n");
          break;
        case '\f':
          sb.append("\\f");
          break;
        case '\r':
          sb.append("\\r");
          break;
        default:
          if (c < ' ' || (c >= '' && c < ' ') || (
            c >= ' ' && c < '℀')) {
            String t = "000" + Integer.toHexString(c);
            sb.append("\\u" + t.substring(t.length() - 4));
            break;
          } 
          sb.append(c);
          break;
      } 
    } 
    sb.append('"');
    return sb.toString();
  }
  
  public Object remove(String key) {
    return this.myHashMap.remove(key);
  }
  
  public Iterator sortedKeys() {
    return (new TreeSet(this.myHashMap.keySet())).iterator();
  }
  
  static void testValidity(Object o) throws JSONException {
    if (o != null)
      if (o instanceof Double) {
        if (((Double)o).isInfinite() || ((Double)o).isNaN())
          throw new JSONException(
              "JSON does not allow non-finite numbers."); 
      } else if (o instanceof Float && ((
        (Float)o).isInfinite() || ((Float)o).isNaN())) {
        throw new JSONException(
            "JSON does not allow non-finite numbers.");
      }  
  }
  
  public JSONArray toJSONArray(JSONArray names) throws JSONException {
    if (names == null || names.length() == 0)
      return null; 
    JSONArray ja = new JSONArray();
    for (int i = 0; i < names.length(); i++)
      ja.put(opt(names.getString(i))); 
    return ja;
  }
  
  public String toString() {
    try {
      Iterator keys = keys();
      StringBuffer sb = new StringBuffer("{");
      while (keys.hasNext()) {
        Object o = keys.next();
        Object v = this.myHashMap.get(o);
        if (v == null || v.equals(null))
          continue; 
        if (sb.length() > 1)
          sb.append(','); 
        sb.append(quote(o.toString()));
        sb.append(':');
        sb.append(_valueToString(v));
      } 
      sb.append('}');
      return sb.toString();
    } catch (Exception e) {
      e.printStackTrace();
      return null;
    } 
  }
  
  static String valueToString(Object value) throws JSONException {
    if (value == null || value.equals(null))
      return "null"; 
    if (value instanceof JSONString) {
      Object o;
      try {
        o = ((JSONString)value).toJSONString();
      } catch (Exception e) {
        throw new JSONException(e);
      } 
      if (o instanceof String)
        return (String)o; 
      throw new JSONException("Bad value from toJSONString: " + o);
    } 
    if (value instanceof Number)
      return numberToString((Number)value); 
    if (value instanceof Boolean || value instanceof com.jiuqi.util.json.JSONObject || 
      value instanceof JSONArray)
      return value.toString(); 
    if (value instanceof Map)
      return (new com.jiuqi.util.json.JSONObject((Map)value)).toString(); 
    if (value instanceof Collection)
      return (new JSONArray((Collection)value)).toString(); 
    if (value.getClass().isArray())
      return (new JSONArray(value)).toString(); 
    return quote(value.toString());
  }
  
  private String _valueToString(Object value) throws JSONException {
    if (this.valueConvertor != null) {
      String s = this.valueConvertor.toString(value);
      if (s != null)
        return s; 
    } 
    return valueToString(value);
  }
  
  public Writer write(Writer writer) throws JSONException {
    try {
      boolean b = false;
      Iterator keys = keys();
      writer.write(123);
      while (keys.hasNext()) {
        if (b)
          writer.write(44); 
        Object k = keys.next();
        writer.write(quote(k.toString()));
        writer.write(58);
        Object v = this.myHashMap.get(k);
        if (v instanceof com.jiuqi.util.json.JSONObject) {
          ((com.jiuqi.util.json.JSONObject)v).write(writer);
        } else if (v instanceof JSONArray) {
          ((JSONArray)v).write(writer);
        } else {
          writer.write(valueToString(v));
        } 
        b = true;
      } 
      writer.write(125);
      return writer;
    } catch (IOException e) {
      throw new JSONException(e);
    } 
  }
}
